@@ -21,9 +21,9 @@ use crate::context::ValueRender;
 // Returns the number of items in an array or an object, or the number of characters in a string.
 pub fn length(value: &Value, _: &HashMap<String, Value>) -> Result<Value> {
     match value {
-        Value::Array(arr) => Ok(to_value(&arr.len()).unwrap()),
-        Value::Object(m) => Ok(to_value(&m.len()).unwrap()),
-        Value::String(s) => Ok(to_value(&s.chars().count()).unwrap()),
+        Value::Array(arr) => Ok(to_value(arr.len()).unwrap()),
+        Value::Object(m) => Ok(to_value(m.len()).unwrap()),
+        Value::String(s) => Ok(to_value(s.chars().count()).unwrap()),
         _ => Err(Error::msg(
             "Filter `length` was used on a value that isn't an array, an object, or a string.",
         )),
@@ -38,7 +38,7 @@ pub fn reverse(value: &Value, _: &HashMap<String, Value>) -> Result<Value> {
             rev.reverse();
             to_value(&rev).map_err(Error::json)
         }
-        Value::String(s) => to_value(&String::from_iter(s.chars().rev())).map_err(Error::json),
+        Value::String(s) => to_value(String::from_iter(s.chars().rev())).map_err(Error::json),
         _ => Err(Error::msg(format!(
             "Filter `reverse` received an incorrect type for arg `value`: \
              got `{}` but expected Array|String",
@@ -74,12 +74,8 @@ pub fn date(value: &Value, args: &HashMap<String, Value>) -> Result<Value> {
         None => "%Y-%m-%d".to_string(),
     };
 
-    let items: Vec<Item> = StrftimeItems::new(&format)
-        .filter(|item| match item {
-            Item::Error => true,
-            _ => false,
-        })
-        .collect();
+    let items: Vec<Item> =
+        StrftimeItems::new(&format).filter(|item| matches!(item, Item::Error)).collect();
     if !items.is_empty() {
         return Err(Error::msg(format!("Invalid date format `{}`", format)));
     }
@@ -102,16 +98,17 @@ pub fn date(value: &Value, args: &HashMap<String, Value>) -> Result<Value> {
         let locale = match args.get("locale") {
             Some(val) => {
                 let locale = try_get_value!("date", "locale", String, val);
-                chrono::Locale::try_from(locale.as_str()).or_else(|_| {
-                    Err(Error::msg(format!("Error parsing `{}` as a locale", locale)))
-                })?
+                chrono::Locale::try_from(locale.as_str())
+                    .map_err(|_| Error::msg(format!("Error parsing `{}` as a locale", locale)))?
             }
             None => chrono::Locale::POSIX,
         };
         match value {
             Value::Number(n) => match n.as_i64() {
                 Some(i) => {
-                    let date = NaiveDateTime::from_timestamp(i, 0);
+                    let date = NaiveDateTime::from_timestamp_opt(i, 0).expect(
+                        "out of bound seconds should not appear, as we set nanoseconds to zero",
+                    );
                     match timezone {
                         Some(timezone) => {
                             timezone.from_utc_datetime(&date).format_localized(&format, locale)
@@ -144,9 +141,14 @@ pub fn date(value: &Value, args: &HashMap<String, Value>) -> Result<Value> {
                         },
                     }
                 } else {
-                    match NaiveDate::parse_from_str(&s, "%Y-%m-%d") {
-                        Ok(val) => DateTime::<Utc>::from_utc(val.and_hms(0, 0, 0), Utc)
-                            .format_localized(&format, locale),
+                    match NaiveDate::parse_from_str(s, "%Y-%m-%d") {
+                        Ok(val) => DateTime::<Utc>::from_utc(
+                            val.and_hms_opt(0, 0, 0).expect(
+                                "out of bound should not appear, as we set the time to zero",
+                            ),
+                            Utc,
+                        )
+                        .format_localized(&format, locale),
                         Err(_) => {
                             return Err(Error::msg(format!(
                                 "Error parsing `{:?}` as YYYY-MM-DD date",
@@ -170,7 +172,9 @@ pub fn date(value: &Value, args: &HashMap<String, Value>) -> Result<Value> {
     let formatted = match value {
         Value::Number(n) => match n.as_i64() {
             Some(i) => {
-                let date = NaiveDateTime::from_timestamp(i, 0);
+                let date = NaiveDateTime::from_timestamp_opt(i, 0).expect(
+                    "out of bound seconds should not appear, as we set nanoseconds to zero",
+                );
                 match timezone {
                     Some(timezone) => timezone.from_utc_datetime(&date).format(&format),
                     None => date.format(&format),
@@ -196,8 +200,13 @@ pub fn date(value: &Value, args: &HashMap<String, Value>) -> Result<Value> {
                     },
                 }
             } else {
-                match NaiveDate::parse_from_str(&s, "%Y-%m-%d") {
-                    Ok(val) => DateTime::<Utc>::from_utc(val.and_hms(0, 0, 0), Utc).format(&format),
+                match NaiveDate::parse_from_str(s, "%Y-%m-%d") {
+                    Ok(val) => DateTime::<Utc>::from_utc(
+                        val.and_hms_opt(0, 0, 0)
+                            .expect("out of bound should not appear, as we set the time to zero"),
+                        Utc,
+                    )
+                    .format(&format),
                     Err(_) => {
                         return Err(Error::msg(format!(
                             "Error parsing `{:?}` as YYYY-MM-DD date",
@@ -216,7 +225,7 @@ pub fn date(value: &Value, args: &HashMap<String, Value>) -> Result<Value> {
         }
     };
 
-    to_value(&formatted.to_string()).map_err(Error::json)
+    to_value(formatted.to_string()).map_err(Error::json)
 }
 
 // Returns the given value as a string.
@@ -238,23 +247,23 @@ mod tests {
     #[test]
     fn as_str_object() {
         let map: HashMap<String, String> = HashMap::new();
-        let result = as_str(&to_value(&map).unwrap(), &HashMap::new());
+        let result = as_str(&to_value(map).unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&"[object]").unwrap());
+        assert_eq!(result.unwrap(), to_value("[object]").unwrap());
     }
 
     #[test]
     fn as_str_vec() {
-        let result = as_str(&to_value(&vec![1, 2, 3, 4]).unwrap(), &HashMap::new());
+        let result = as_str(&to_value(vec![1, 2, 3, 4]).unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&"[1, 2, 3, 4]").unwrap());
+        assert_eq!(result.unwrap(), to_value("[1, 2, 3, 4]").unwrap());
     }
 
     #[test]
     fn length_vec() {
-        let result = length(&to_value(&vec![1, 2, 3, 4]).unwrap(), &HashMap::new());
+        let result = length(&to_value(vec![1, 2, 3, 4]).unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&4).unwrap());
+        assert_eq!(result.unwrap(), to_value(4).unwrap());
     }
 
     #[test]
@@ -263,46 +272,46 @@ mod tests {
         map.insert("foo".to_string(), "bar".to_string());
         let result = length(&to_value(&map).unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&1).unwrap());
+        assert_eq!(result.unwrap(), to_value(1).unwrap());
     }
 
     #[test]
     fn length_str() {
-        let result = length(&to_value(&"Hello World").unwrap(), &HashMap::new());
+        let result = length(&to_value("Hello World").unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&11).unwrap());
+        assert_eq!(result.unwrap(), to_value(11).unwrap());
     }
 
     #[test]
     fn length_str_nonascii() {
-        let result = length(&to_value(&"日本語").unwrap(), &HashMap::new());
+        let result = length(&to_value("日本語").unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&3).unwrap());
+        assert_eq!(result.unwrap(), to_value(3).unwrap());
     }
 
     #[test]
     fn length_num() {
-        let result = length(&to_value(&15).unwrap(), &HashMap::new());
+        let result = length(&to_value(15).unwrap(), &HashMap::new());
         assert!(result.is_err());
     }
 
     #[test]
     fn reverse_vec() {
-        let result = reverse(&to_value(&vec![1, 2, 3, 4]).unwrap(), &HashMap::new());
+        let result = reverse(&to_value(vec![1, 2, 3, 4]).unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&vec![4, 3, 2, 1]).unwrap());
+        assert_eq!(result.unwrap(), to_value(vec![4, 3, 2, 1]).unwrap());
     }
 
     #[test]
     fn reverse_str() {
-        let result = reverse(&to_value(&"Hello World").unwrap(), &HashMap::new());
+        let result = reverse(&to_value("Hello World").unwrap(), &HashMap::new());
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), to_value(&"dlroW olleH").unwrap());
+        assert_eq!(result.unwrap(), to_value("dlroW olleH").unwrap());
     }
 
     #[test]
     fn reverse_num() {
-        let result = reverse(&to_value(&1.23).unwrap(), &HashMap::new());
+        let result = reverse(&to_value(1.23).unwrap(), &HashMap::new());
         assert!(result.is_err());
         assert_eq!(
             result.err().unwrap().to_string(),
