@@ -2,7 +2,7 @@ use std::collections::HashMap;
 
 use lazy_static::lazy_static;
 use pest::iterators::Pair;
-use pest::prec_climber::{Assoc, Operator, PrecClimber};
+use pest::pratt_parser::{Assoc, Op, PrattParser};
 use pest::Parser;
 use pest_derive::Parser;
 
@@ -27,24 +27,19 @@ use self::ast::*;
 pub use self::whitespace::remove_whitespace;
 
 lazy_static! {
-    static ref MATH_CLIMBER: PrecClimber<Rule> = PrecClimber::new(vec![
-        // +, -
-        Operator::new(Rule::op_plus, Assoc::Left) | Operator::new(Rule::op_minus, Assoc::Left),
-        // *, /, %
-        Operator::new(Rule::op_times, Assoc::Left) |
-        Operator::new(Rule::op_slash, Assoc::Left) |
-        Operator::new(Rule::op_modulo, Assoc::Left),
-    ]);
-    static ref COMPARISON_EXPR_CLIMBER: PrecClimber<Rule> = PrecClimber::new(vec![
-        // <, <=, >, >=, ==, !=
-        Operator::new(Rule::op_lt, Assoc::Left) | Operator::new(Rule::op_lte, Assoc::Left)
-        | Operator::new(Rule::op_gt, Assoc::Left) | Operator::new(Rule::op_gte, Assoc::Left)
-        | Operator::new(Rule::op_eq, Assoc::Left) | Operator::new(Rule::op_ineq, Assoc::Left),
-    ]);
-    static ref LOGIC_EXPR_CLIMBER: PrecClimber<Rule> = PrecClimber::new(vec![
-        Operator::new(Rule::op_or, Assoc::Left),
-        Operator::new(Rule::op_and, Assoc::Left),
-    ]);
+    static ref MATH_PARSER: PrattParser<Rule> = PrattParser::new()
+        .op(Op::infix(Rule::op_plus, Assoc::Left) | Op::infix(Rule::op_minus, Assoc::Left)) // +, -
+        .op(Op::infix(Rule::op_times, Assoc::Left)
+            | Op::infix(Rule::op_slash, Assoc::Left)
+            | Op::infix(Rule::op_modulo, Assoc::Left)); // *, /, %
+
+    static ref COMPARISON_EXPR_PARSER: PrattParser<Rule> = PrattParser::new()
+        .op(Op::infix(Rule::op_lt, Assoc::Left) | Op::infix(Rule::op_lte, Assoc::Left) | Op::infix(Rule::op_gt, Assoc::Left)
+            | Op::infix(Rule::op_gte, Assoc::Left)
+            | Op::infix(Rule::op_eq, Assoc::Left)| Op::infix(Rule::op_ineq, Assoc::Left)); // <, <=, >, >=, ==, !=
+
+    static ref LOGIC_EXPR_PARSER: PrattParser<Rule> = PrattParser::new()
+        .op(Op::infix(Rule::op_or, Assoc::Left)).op(Op::infix(Rule::op_and, Assoc::Left));
 }
 
 /// Strings are delimited by double quotes, single quotes and backticks
@@ -222,7 +217,7 @@ fn parse_string_concat(pair: Pair<Rule>) -> TeraResult<ExprVal> {
 }
 
 fn parse_basic_expression(pair: Pair<Rule>) -> TeraResult<ExprVal> {
-    let primary = |pair| parse_basic_expression(pair);
+    let primary = parse_basic_expression;
 
     let infix = |lhs: TeraResult<ExprVal>, op: Pair<Rule>, rhs: TeraResult<ExprVal>| {
         Ok(ExprVal::Math(MathExpr {
@@ -266,7 +261,9 @@ fn parse_basic_expression(pair: Pair<Rule>) -> TeraResult<ExprVal> {
         Rule::fn_call => ExprVal::FunctionCall(parse_fn_call(pair)?),
         Rule::macro_call => ExprVal::MacroCall(parse_macro_call(pair)?),
         Rule::dotted_square_bracket_ident => ExprVal::Ident(pair.as_str().to_string()),
-        Rule::basic_expr => MATH_CLIMBER.climb(pair.into_inner(), primary, infix)?,
+        Rule::basic_expr => {
+            MATH_PARSER.map_primary(primary).map_infix(infix).parse(pair.into_inner())?
+        }
         _ => unreachable!("Got {:?} in parse_basic_expression: {}", pair.as_rule(), pair.as_str()),
     };
     Ok(expr)
@@ -362,7 +359,7 @@ fn parse_in_condition(pair: Pair<Rule>) -> TeraResult<Expr> {
 
 /// A basic expression with optional filters with prece
 fn parse_comparison_val(pair: Pair<Rule>) -> TeraResult<Expr> {
-    let primary = |pair| parse_comparison_val(pair);
+    let primary = parse_comparison_val;
 
     let infix = |lhs: TeraResult<Expr>, op: Pair<Rule>, rhs: TeraResult<Expr>| {
         Ok(Expr::new(ExprVal::Math(MathExpr {
@@ -381,14 +378,16 @@ fn parse_comparison_val(pair: Pair<Rule>) -> TeraResult<Expr> {
 
     let expr = match pair.as_rule() {
         Rule::basic_expr_filter => parse_basic_expr_with_filters(pair)?,
-        Rule::comparison_val => MATH_CLIMBER.climb(pair.into_inner(), primary, infix)?,
+        Rule::comparison_val => {
+            MATH_PARSER.map_primary(primary).map_infix(infix).parse(pair.into_inner())?
+        }
         _ => unreachable!("Got {:?} in parse_comparison_val", pair.as_rule()),
     };
     Ok(expr)
 }
 
 fn parse_comparison_expression(pair: Pair<Rule>) -> TeraResult<Expr> {
-    let primary = |pair| parse_comparison_expression(pair);
+    let primary = parse_comparison_expression;
 
     let infix = |lhs: TeraResult<Expr>, op: Pair<Rule>, rhs: TeraResult<Expr>| {
         Ok(Expr::new(ExprVal::Logic(LogicExpr {
@@ -410,7 +409,7 @@ fn parse_comparison_expression(pair: Pair<Rule>) -> TeraResult<Expr> {
         Rule::comparison_val => parse_comparison_val(pair)?,
         Rule::string_expr_filter => parse_string_expr_with_filters(pair)?,
         Rule::comparison_expr => {
-            COMPARISON_EXPR_CLIMBER.climb(pair.into_inner(), primary, infix)?
+            COMPARISON_EXPR_PARSER.map_primary(primary).map_infix(infix).parse(pair.into_inner())?
         }
         _ => unreachable!("Got {:?} in parse_comparison_expression", pair.as_rule()),
     };
@@ -438,7 +437,7 @@ fn parse_logic_val(pair: Pair<Rule>) -> TeraResult<Expr> {
 }
 
 fn parse_logic_expr(pair: Pair<Rule>) -> TeraResult<Expr> {
-    let primary = |pair: Pair<Rule>| parse_logic_expr(pair);
+    let primary = parse_logic_expr;
 
     let infix = |lhs: TeraResult<Expr>, op: Pair<Rule>, rhs: TeraResult<Expr>| match op.as_rule() {
         Rule::op_or => Ok(Expr::new(ExprVal::Logic(LogicExpr {
@@ -459,7 +458,9 @@ fn parse_logic_expr(pair: Pair<Rule>) -> TeraResult<Expr> {
 
     let expr = match pair.as_rule() {
         Rule::logic_val => parse_logic_val(pair)?,
-        Rule::logic_expr => LOGIC_EXPR_CLIMBER.climb(pair.into_inner(), primary, infix)?,
+        Rule::logic_expr => {
+            LOGIC_EXPR_PARSER.map_primary(primary).map_infix(infix).parse(pair.into_inner())?
+        }
         _ => unreachable!("Got {:?} in parse_logic_expr", pair.as_rule()),
     };
     Ok(expr)
@@ -765,7 +766,7 @@ fn parse_macro_arg(p: Pair<Rule>) -> TeraResult<ExprVal> {
             "False" => Some(ExprVal::Bool(false)),
             _ => unreachable!(),
         },
-        Rule::string => Some(ExprVal::String(replace_string_markers(&p.as_str()))),
+        Rule::string => Some(ExprVal::String(replace_string_markers(p.as_str()))),
         _ => unreachable!("Got {:?} in parse_macro_arg: {}", p.as_rule(), p.as_str()),
     };
 
@@ -1054,7 +1055,6 @@ fn parse_content(pair: Pair<Rule>) -> TeraResult<Vec<Node>> {
             Rule::set_global_tag => nodes.push(parse_set_tag(p, true)?),
             Rule::raw => nodes.push(parse_raw_tag(p)),
             Rule::variable_tag => nodes.push(parse_variable_tag(p)?),
-            Rule::macro_definition => nodes.push(parse_macro_definition(p)?),
             Rule::forloop => nodes.push(parse_forloop(p)?),
             Rule::break_tag => nodes.push(parse_break_tag(p)),
             Rule::continue_tag => nodes.push(parse_continue_tag(p)),
@@ -1129,7 +1129,7 @@ pub fn parse(input: &str) -> TeraResult<Vec<Node>> {
                     Rule::test_not => "a negated test".to_string(),
                     Rule::test_call => "a test call".to_string(),
                     Rule::test_arg => "a test argument (any expressions including arrays)".to_string(),
-                    Rule::test_args => "a list of test arguments (any expression including arrayss)".to_string(),
+                    Rule::test_args => "a list of test arguments (any expression including arrays)".to_string(),
                     Rule::macro_fn | Rule::macro_fn_wrapper => "a macro function".to_string(),
                     Rule::macro_call => "a macro function call".to_string(),
                     Rule::macro_def_arg => {
@@ -1176,7 +1176,7 @@ pub fn parse(input: &str) -> TeraResult<Vec<Node>> {
                     | Rule::macro_if
                     | Rule::for_if
                     | Rule::filter_section_if => {
-                        "a `if` tag".to_string()
+                        "an `if` tag".to_string()
                     }
                     Rule::elif_tag => "an `elif` tag".to_string(),
                     Rule::else_tag => "an `else` tag".to_string(),
@@ -1213,6 +1213,7 @@ pub fn parse(input: &str) -> TeraResult<Vec<Node>> {
             Rule::extends_tag => nodes.push(parse_extends(p)),
             Rule::import_macro_tag => nodes.push(parse_import_macro(p)),
             Rule::content => nodes.extend(parse_content(p)?),
+            Rule::macro_definition => nodes.push(parse_macro_definition(p)?),
             Rule::comment_tag => (),
             Rule::EOI => (),
             _ => unreachable!("unknown tpl rule: {:?}", p.as_rule()),
