@@ -18,7 +18,7 @@ fn render_template(content: &str, context: &Context) -> Result<String> {
     let mut tera = Tera::default();
     tera.add_raw_template("hello.html", content).unwrap();
     tera.register_function("get_number", |_: &HashMap<String, Value>| Ok(Value::Number(10.into())));
-    tera.register_function("get_true", |_: &HashMap<String, Value>| Ok(Value::Bool(true.into())));
+    tera.register_function("get_true", |_: &HashMap<String, Value>| Ok(Value::Bool(true)));
     tera.register_function("get_string", |_: &HashMap<String, Value>| {
         Ok(Value::String("Hello".to_string()))
     });
@@ -36,7 +36,7 @@ fn render_simple_string() {
 fn render_variable_block_lit_expr() {
     let inputs = vec![
         ("{{ 1 }}", "1"),
-        ("{{ 3.14 }}", "3.14"),
+        ("{{ 3.18 }}", "3.18"),
         ("{{ \"hey\" }}", "hey"),
         (r#"{{ "{{ hey }}" }}"#, "{{ hey }}"),
         ("{{ true }}", "true"),
@@ -696,21 +696,21 @@ fn can_do_string_concat() {
     context.insert("a_string", "hello");
     context.insert("another_string", "xXx");
     context.insert("an_int", &1);
-    context.insert("a_float", &3.14);
+    context.insert("a_float", &3.18);
 
     let inputs = vec![
         (r#"{{ "hello" ~ " world" }}"#, "hello world"),
         (r#"{{ "hello" ~ 1 }}"#, "hello1"),
-        (r#"{{ "hello" ~ 3.14 }}"#, "hello3.14"),
-        (r#"{{ 3.14 ~ "hello"}}"#, "3.14hello"),
+        (r#"{{ "hello" ~ 3.18 }}"#, "hello3.18"),
+        (r#"{{ 3.18 ~ "hello"}}"#, "3.18hello"),
         (r#"{{ "hello" ~ get_string() }}"#, "helloHello"),
         (r#"{{ get_string() ~ "hello" }}"#, "Hellohello"),
-        (r#"{{ get_string() ~ 3.14 }}"#, "Hello3.14"),
+        (r#"{{ get_string() ~ 3.18 }}"#, "Hello3.18"),
         (r#"{{ a_string ~ " world" }}"#, "hello world"),
         (r#"{{ a_string ~ ' world ' ~ another_string }}"#, "hello world xXx"),
         (r#"{{ a_string ~ another_string }}"#, "helloxXx"),
         (r#"{{ a_string ~ an_int }}"#, "hello1"),
-        (r#"{{ a_string ~ a_float }}"#, "hello3.14"),
+        (r#"{{ a_string ~ a_float }}"#, "hello3.18"),
     ];
 
     for (input, expected) in inputs {
